#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <cstdlib>
#include "TLorentzVector.h"

using namespace std;

/*
   Monte Carlo integration.
   We calculate the x-section for p p -> Higgs  + X
   and plot the distribution in pt and eta
   
   Authors: H. Jung, 
            A. Bermudez Martinez, L.I. Estevez Banos, J. Lidrych, M. Mendizabal Morentin, 
            S. Taheri Monfared, P. L.S. Connor, Q. Wang, H. Yang, R. Zlebcik 


*/


double sigma (double m2, double sh, double th)
{	
    // calculate sigma0 (qq -> gamma^*)
    // double aem=1./137.;
    // result = 4.*pi*pi*aem/3.;

    // calculate sigma0(gg->H) 
    // sigma0 = as^2/pi /576 * GF * sqrt(2)*\Delta(1.-tau) 
    const double as = 0.1;   //alphaS
    const double GF = 1.166E-5; //Fermi constant
    double result = as * as * sqrt(2.) * GF /M_PI / 576. ;
    //      cout << " M = " << sqrt(m2) << endl;
    return result; 
}


void gauss2D (double sigma, double &kx, double &ky)
{
    double kT  = sigma * sqrt(-2*log(Rand()));
    double phi = 2*M_PI * Rand();
    kx = kT*cos(phi);
    ky = kT*sin(phi);
}


void getpdf (double xmin, double q2, double& weightx, double& x, double& kx, double& ky)
{
    double xmax = 0.999;
/*
		generate here x according to a 1/x distribution
            give a weight according a simple pdf ala
            xg(x) = 3(1-x)^5. 
// 		note, we want here the pdf, not x*pdf.
            
            caldulate kt2 from the gauss distribution with
            mean = 0 and width = 0,7  in kt2 
            use a random azimuthal angle phi to get k_x,k_y
            
            give back weightx, kx, ky and kt2
            
            
*/            
/* 

   insert here your code
   
*/

}

int main (int argc,char **argv)
{
    TH1::SetDefaultSumw2();
    TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
    const int npoints = 1.0e6;
    const double x1min = 0.00001, x2min = 0.00001;
    const double m_higgs = 125., Gamma = 0.4; /* Width from PDG 2014 */
    const double s = 4*3500.*3500.;

    const double gev2nb = 0.3893857E+6;

    // book the histogram TH1D("label","title",nr of bins, xlow,xhigh )
    TH1D *histo1  = new TH1D("x1","x1",100, -5, 0.);
    TH1D *histo2  = new TH1D("x2","x2",100, -5, 0.);
    TH1D *histo3  = new TH1D("kt1 ","kt1 ",100, 0, 10.);
    TH1D *histo4  = new TH1D("kt2 ","kt2 ",100, 0, 10.);
    TH1D *histo5  = new TH1D("pt ","pt ",50, 0, 10.);
    TH1D *histo6  = new TH1D("eta ","eta",50, -8, 8.);
    TH1D *histo7  = new TH1D("Mass ","mass",50, 60., 160.);

    // initialise random number generator: rlxd_init( luxory level, seed )
    rlxd_init(2,32767);

    const double q2 = 10000;
    const double mass_min = 0.;
    const double mass_max = 200.;

    double sum0 = 0, sum00 = 0;
    int nacc = 0;

    for (int n1 = 0; n1 < npoints; ++n1) {
/*
  this is using importance sampling
  generate starting distribution in x and kt
  for first incoming parton
  calculate px,py,pz, E and kt2
  use the four-vector struct declaration 
  
  generate starting distribution in x and kt
  for second incoming parton
  calculate px,py,pz, E and kt2
  use the four-vector struct declaration 
  
  calculate the mass of the resulting system (pa+pb)^2
  calculate the pdfs at x1,q2 and x2,q2, with q2=500
  
  calculate the cross section 
  xsec=pdf(x1,q2)*pdf(x2,q2)*sigma*gev2nb
  the pdf should be included in the weight for x
  
  calculate the integral and its uncertainty
  
  plot the distribution in x1,x2 versus log10(x)
  which Jacobian is needed to get the proper distribution ?

*/         
/*

  insert here your code
 
*/  

    }
    //                                  
    sum0  /= npoints;
    sum00 /= npoints;
    double sigma2 = sum00 - sum0*sum0 ;
    double error = sqrt(sigma2/npoints) ;
    cout << " nr of events accepted: "<< nacc << endl;
    cout<< " integral for Higgs xsection is [pb]: " << sum0 * 1000.<< " +/- " << error*1000.<< endl;

    // write histogramm out to file
    TFile file("output-example8.root","RECREATE");
    //general root settings 
    gROOT->Reset();
    gROOT->SetStyle("Plain");
    gStyle->SetPadTickY(1); // ticks at right side
    gStyle->SetOptStat(0); // get rid of statistics box
    TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
    // divide the canvas in 1 parts in x and 1 in y
    c->Divide(3,3);
    c->cd(1);
    histo1->Scale(1./npoints, "width");
    histo1->Draw();
    c->cd(2);
    histo2->Scale(1./npoints, "width");
    histo2->Draw();
    c->cd(3);
    histo3->Draw();
    c->cd(4);
    histo4->Draw();
    c->cd(5);
    histo5->Draw();
    c->cd(6);
    histo6->Draw();
    c->cd(7);
    histo7->Draw();
    c-> Draw();
    c-> Print("example8.pdf");

    histo1->Write();
    histo2->Write();
    histo3->Write();
    histo4->Write();
    histo5->Write();
    histo6->Write();
    histo7->Write();
    file.Close();
    gMyRootApp->Run();
    return EXIT_SUCCESS;
}
